"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fixDependencies = fixDependencies;
var _helpers = require("./helpers");
var _path = require("../helpers/path");
var dependencyTypes = ['dependencies', 'devDependencies'];
var logRed = function logRed(msg) {
  return console.log("\x1B[31m".concat(msg, "\x1B[0m"));
};
var logYellow = function logYellow(msg) {
  return console.log("\x1B[33m".concat(msg, "\x1B[0m"));
};
var installCmd = {
  dependencies: 'npm install --save --save-exact',
  devDependencies: 'npm install --save-dev --save-exact'
};

/*
  Given a `checkDependencies` object in props, either fix installs
  (if --fix-deps is set) or throw an error. Also fails if there are
  any conflicts.
*/
function fixDependencies(_ref) {
  var config = _ref.config,
    packageJson = _ref.packageJson,
    props = _ref.props;
  var autofix = props.argv.includes('--fix-dependencies');
  var monorepoPath = (0, _path.join)(config.path, 'package.json');
  var monorepo = JSON.parse(_helpers.fs.readFileSync(monorepoPath, 'utf8'));
  var needInstall = false;
  var hasConflicts = false;
  var checkDependencies = props.checkDependencies;
  if (!checkDependencies) {
    throw new Error("No 'checkDependencies' in props. Missing 'checkDependencies' action ?");
  }
  dependencyTypes.forEach(function (dependencyType) {
    var _checkDependencies$de = checkDependencies[dependencyType],
      toInstall = _checkDependencies$de.toInstall,
      conflict = _checkDependencies$de.conflict;
    if (conflict.length) {
      hasConflicts = true;
      logRed("".concat(dependencyType, " CONFLICT"));
      conflict.forEach(function (c) {
        return console.log(JSON.stringify(c, null, 2));
      });
    }
    if (toInstall.length) {
      needInstall = true;
      if (autofix) {
        // Update monorepo dependency information
        var dependencies = monorepo[dependencyType] || {};
        toInstall.forEach(function (info) {
          dependencies[info.dependency] = info.version;
        });
        monorepo[dependencyType] = dependencies;
        logYellow("".concat(dependencyType, " UPDATE"));
        console.log(toInstall.map(function (info) {
          return "".concat(info.dependency, "@").concat(info.version);
        }));
      } else {
        logRed("".concat(dependencyType, " TO INSTALL"));
        console.log("".concat(installCmd[dependencyType], " ").concat(toInstall.map(function (info) {
          return "".concat(info.dependency, "@").concat(info.version);
        }).join(' ')));
      }
    }
  });
  if (hasConflicts || needInstall && !autofix) {
    var msgs = [];
    if (needInstall) {
      msgs.push('need install');
    }
    if (hasConflicts) {
      msgs.push('have conflicts');
    }
    throw new Error("Dependencies ".concat(msgs.join(' and '), "."));
  }
  if (needInstall && autofix) {
    _helpers.fs.writeFileSync(monorepoPath, JSON.stringify(monorepo, null, 2) + '\n', 'utf-8');
  }
  return {
    fixDependencies: needInstall ? 'fixed' : 'noop'
  };
}
//# sourceMappingURL=fixDependencies.js.map